/******************************************************************************
* Copyright (c) 2022 Xilinx, Inc.  All rights reserved.
* Copyright (c) 2022 - 2023 Advanced Micro Devices, Inc. All Rights Reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/

/*****************************************************************************/
/**
*
* @file psmx_global.h
*
* This file contains PSM Global definitions used by PSM Firmware
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver	Who		Date		Changes
* ---- ---- -------- ------------------------------
* 1.00  sr   03/03/2022 Initial release
*
* </pre>
*
* @note
*
******************************************************************************/

#ifndef XPSMFW_PSMX_GLOBAL_H_
#define XPSMFW_PSMX_GLOBAL_H_


#ifdef __cplusplus
extern "C" {
#endif

/**
 * PSMX_GLOBAL_REG Base Address
 */
#define PSMX_GLOBAL_REG_BASEADDR      ((u32)0xEBC90000U)

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_CNTRL
 */
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000000U )
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FULLMASK     ((u32)0x0003ff12U)
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FULLRWMASK  (u32)0x0000ff12
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_DEFVAL   (u32)0x8800

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_DBG_WAKE_SHIFT   17
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_DBG_WAKE_WIDTH   1
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_DBG_WAKE_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_DBG_WAKE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_SLEEP_SHIFT   16
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_SLEEP_WIDTH   1
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_SLEEP_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_MB_SLEEP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_WRITE_QOS_SHIFT   12
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_WRITE_QOS_WIDTH   4
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_WRITE_QOS_MASK    (u32)0x0000f000
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_WRITE_QOS_DEFVAL  (u32)0x8

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_READ_QOS_SHIFT   8
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_READ_QOS_WIDTH   4
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_READ_QOS_MASK    (u32)0x00000f00
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_READ_QOS_DEFVAL  (u32)0x8

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FW_IS_PRESENT_SHIFT   4
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FW_IS_PRESENT_WIDTH   1
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FW_IS_PRESENT_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_FW_IS_PRESENT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_SLVERR_ENABLE_SHIFT   1
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_SLVERR_ENABLE_WIDTH   1
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_SLVERR_ENABLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GLOBAL_CNTRL_SLVERR_ENABLE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_ADDR_ERROR_STATUS
 */
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000010 )
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_STATUS_SHIFT   0
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_STATUS_WIDTH   1
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_STATUS_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_STATUS_STATUS_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK
 */
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000014 )
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_MASK_SHIFT   0
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_MASK_WIDTH   1
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_MASK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_MASK_MASK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN
 */
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000018 )
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_ENABLE_SHIFT   0
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_ENABLE_WIDTH   1
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_ENABLE_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_EN_ENABLE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS
 */
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000001C )
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_DISABLE_SHIFT   0
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_DISABLE_WIDTH   1
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_DISABLE_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_ADDR_ERROR_INT_DIS_DISABLE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PS_SW_ERR
 */
#define PSMX_GLOBAL_REG_PS_SW_ERR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000020 )
#define PSMX_GLOBAL_REG_PS_SW_ERR_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PS_SW_ERR_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PS_SW_ERR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PS_SW_ERR_NCR_FLAG_SHIFT   31
#define PSMX_GLOBAL_REG_PS_SW_ERR_NCR_FLAG_WIDTH   1
#define PSMX_GLOBAL_REG_PS_SW_ERR_NCR_FLAG_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PS_SW_ERR_NCR_FLAG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PS_SW_ERR_CR_FLAG_SHIFT   30
#define PSMX_GLOBAL_REG_PS_SW_ERR_CR_FLAG_WIDTH   1
#define PSMX_GLOBAL_REG_PS_SW_ERR_CR_FLAG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PS_SW_ERR_CR_FLAG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PS_SW_ERR_DATA_SHIFT   0
#define PSMX_GLOBAL_REG_PS_SW_ERR_DATA_WIDTH   30
#define PSMX_GLOBAL_REG_PS_SW_ERR_DATA_MASK    (u32)0x3fffffff
#define PSMX_GLOBAL_REG_PS_SW_ERR_DATA_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR
 */
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000024 )
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_NCR_FLAG_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_NCR_FLAG_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_NCR_FLAG_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_NCR_FLAG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_CR_FLAG_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_CR_FLAG_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_CR_FLAG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_CR_FLAG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_DATA_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_DATA_WIDTH   30
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_DATA_MASK    (u32)0x3fffffff
#define PSMX_GLOBAL_REG_PSM_BOOT_SERV_ERR_DATA_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000030 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE0_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000034 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE1_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000038 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE2_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000003C )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE3_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000040 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE4_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000044 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE5_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000048 )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE6_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7
 */
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000004C )
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_GLOBAL_GEN_STORAGE7_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000050 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE0_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000054 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE1_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000058 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE2_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000005C )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE3_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000060 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE4_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000064 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE5_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000068 )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE6_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7
 */
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000006C )
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_PERS_GLOB_GEN_STORAGE7_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_STATE0
 */
#define PSMX_GLOBAL_REG_PWR_STATE0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000100 )
#define PSMX_GLOBAL_REG_PWR_STATE0_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_STATE0_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_STATE0_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I3_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I3_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I2_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I2_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I1_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I0_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B1_I0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I3_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I3_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I2_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I2_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I1_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I1_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I0_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I0_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_STATE0_OCM_B0_I0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB1_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB0_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMB0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA1_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA0_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_STATE0_TCMA0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE1_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE1_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE0_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE0_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_B_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE1_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE0_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_STATE0_RPU_A_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_STATE0_APU3_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_STATE0_APU2_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_STATE0_APU1_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_STATE0_APU0_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PWR_STATE1
 */
#define PSMX_GLOBAL_REG_PWR_STATE1    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000104 )
#define PSMX_GLOBAL_REG_PWR_STATE1_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_PWR_STATE1_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_STATE1_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_PWR_STATE1_FP_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_STATE1_FP_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE1_FP_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_STATE1_FP_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE1_GEM1_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_STATE1_GEM0_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_STATE1_GEM0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_AUX_PWR_STATE
 */
#define PSMX_GLOBAL_REG_AUX_PWR_STATE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000108 )
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I3_SHIFT   31
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I3_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I2_SHIFT   30
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I2_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I1_SHIFT   29
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I0_SHIFT   28
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B1_I0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I3_SHIFT   27
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I3_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I2_SHIFT   26
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I2_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I1_SHIFT   25
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I1_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I0_SHIFT   24
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I0_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_OCM_B0_I0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB1_SHIFT   23
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB0_SHIFT   22
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMB0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA1_SHIFT   21
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA0_SHIFT   20
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_TCMA0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE1_SHIFT   19
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE1_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE0_SHIFT   18
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE0_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE1_SHIFT   17
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE0_SHIFT   16
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_AUX_PWR_STATE_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000110 )
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_3Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1H_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_1Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_ON_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_ON_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_ON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_L3_ON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP0_STATUS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000114 )
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_RSVD0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_LLC_MODE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_FP_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_3Q_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1H_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_1Q_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_ON_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_ON_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_ON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_L3_ON_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU3_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU2_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU1_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_MASK_APU0_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000118 )
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_3Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1H_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_1Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_ON_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_ON_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_ON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_L3_ON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_EN_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000011C )
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_3Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1H_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_1Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_ON_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_ON_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_ON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_L3_ON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP0_INT_DIS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000120 )
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_3Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1H_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_1Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_ON_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_ON_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_ON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_L3_ON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP0_TRIG_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000124 )
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_FULLMASK     (u32)0x30f00fff
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP1_STATUS_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000128 )
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_FULLMASK     (u32)0x30f00fff
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_DEFVAL   (u32)0x30f00fff

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_GEM0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM1A_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_TCM0A_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_OCM_ISLAND0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_B_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_MASK_RPU_A_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000012C )
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_FULLMASK     (u32)0x30f00fff
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_EN_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000130 )
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_FULLMASK     (u32)0x30f00fff
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP1_INT_DIS_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000134 )
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_FULLMASK     (u32)0x30f00fff
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRUP1_TRIG_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000210 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_3Q_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_3Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_3Q_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_3Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1H_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1H_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1H_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1H_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1Q_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1Q_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1Q_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_1Q_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_OFF_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_OFF_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_OFF_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_L3_OFF_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_STATUS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000214 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_RSVD0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_LLC_MODE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_FP_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE3_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE3_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE2_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE2_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE1_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE0_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_L3_MODE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU3_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU2_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU1_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_MASK_APU0_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000218 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE3_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE3_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE2_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE2_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE1_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE0_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_L3_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_EN_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000021C )
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE3_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE3_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE2_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE2_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE1_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE0_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_L3_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_INT_DIS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000220 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD6_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD6_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD5_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD5_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD4_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD4_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD3_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD3_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD2_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD2_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD1_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD1_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD0_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD0_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE3_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE3_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE2_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE2_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_LLC_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FP_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FP_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE3_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE3_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE2_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE2_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE1_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE1_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE0_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE0_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_L3_MODE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN0_TRIG_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000224 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_FULLMASK     (u32)0x3fffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_RET_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_RET_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_RET_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_RET_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_RET_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_RET_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_RET_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_RET_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_RET_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_RET_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_RET_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_RET_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_RET_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_RET_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_RET_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_RET_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_RET_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_RET_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_RET_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_RET_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_RET_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_RET_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_RET_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_RET_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_STATUS_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000228 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_FULLMASK     (u32)0x3fffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_DEFVAL   (u32)0x3fffffff

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_GEM0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_RET_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_RET_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_RET_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_RET_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_RET_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_RET_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_RET_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_RET_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM1A_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_TCM0A_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_RET_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_RET_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_RET_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_RET_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_RET_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_RET_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_RET_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_RET_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_RET_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_RET_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_RET_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_RET_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_RET_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_RET_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_RET_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_RET_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_RET_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_OCM_ISLAND0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_B_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_MASK_RPU_A_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000022C )
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_FULLMASK     (u32)0x3fffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_RET_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_RET_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_RET_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_RET_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_RET_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_RET_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_RET_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_RET_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_RET_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_RET_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_RET_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_RET_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_RET_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_RET_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_RET_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_RET_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_RET_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_RET_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_RET_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_RET_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_RET_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_RET_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_RET_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_RET_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_EN_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000230 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_FULLMASK     (u32)0x3fffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_RET_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_RET_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_RET_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_RET_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_RET_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_RET_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_RET_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_RET_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_RET_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_RET_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_RET_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_RET_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_RET_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_RET_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_RET_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_RET_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_RET_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_RET_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_RET_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_RET_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_RET_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_RET_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_RET_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_RET_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_INT_DIS_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000234 )
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_FULLMASK     (u32)0x3fffffff
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM1_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM1_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM0_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM0_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_GEM0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_RET_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_RET_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_RET_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_RET_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_RET_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_RET_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_RET_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_RET_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM1A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_TCM0A_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_RET_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_RET_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_RET_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_RET_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_RET_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_RET_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_RET_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_RET_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_RET_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_RET_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_RET_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_RET_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_RET_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_RET_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_RET_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_RET_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_RET_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_RET_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_OCM_ISLAND0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_PWRDWN1_TRIG_RPU_A_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_ISO_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000310 )
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FP_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FP_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_STATUS_FP_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_ISO_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000314 )
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FP_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FP_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_MASK_FP_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_ISO_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000318 )
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FP_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FP_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_EN_FP_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_ISO_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000031C )
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FP_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FP_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_INT_DIS_FP_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_ISO_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000320 )
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FP_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FP_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_ISO_TRIG_FP_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_SWRST_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000410 )
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD1_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD1_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD0_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD0_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FP_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FP_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_LP_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_LP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_LP_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_LP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CLUSTER_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CLUSTER_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CLUSTER_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CLUSTER_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_SWRST_STATUS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000414 )
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD1_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD1_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD0_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD0_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RSVD0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FP_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FP_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_FP_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_LP_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_LP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_LP_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_LP_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CLUSTER_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CLUSTER_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CLUSTER_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CLUSTER_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_B_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_RPU_A_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU3_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU2_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU1_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_MASK_APU0_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_SWRST_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000418 )
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD1_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD1_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD0_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD0_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FP_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FP_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_LP_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_LP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_LP_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_LP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CLUSTER_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CLUSTER_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CLUSTER_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CLUSTER_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_EN_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000041C )
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD1_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD1_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD0_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD0_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FP_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FP_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_LP_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_LP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_LP_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_LP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CLUSTER_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CLUSTER_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CLUSTER_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CLUSTER_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_SWRST_INT_DIS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_SWRST_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000420 )
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD1_SHIFT   31
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD1_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD0_SHIFT   30
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD0_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FP_SHIFT   29
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FP_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_FP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_LP_SHIFT   28
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_LP_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_LP_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_LP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_SHIFT   27
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CLUSTER_SHIFT   26
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CLUSTER_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CLUSTER_SHIFT   25
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CLUSTER_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU_SHIFT   20
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_SWRST_TRIG_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_AUX_STATUS
 */
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000510 )
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_FULLMASK     (u32)0x000334cf
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_10_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_10_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_10_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_9_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_9_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_9_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_8_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_8_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_8_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_7_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_7_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_5_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_5_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_4_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_4_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_AUX_STATUS_SERV_REQ_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_AUX_INT_MASK
 */
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000514 )
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_FULLMASK     (u32)0x000334cf
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_DEFVAL   (u32)0x334cf

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_10_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_10_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_10_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_9_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_9_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_9_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_8_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_8_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_8_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_7_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_7_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_5_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_5_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_4_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_4_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_AUX_INT_MASK_SERV_REQ_0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_REQ_AUX_INT_EN
 */
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000518 )
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_FULLMASK     (u32)0x000334cf
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_10_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_10_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_10_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_9_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_9_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_9_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_8_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_8_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_8_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_7_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_7_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_5_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_5_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_4_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_4_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_AUX_INT_EN_SERV_REQ_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_AUX_INT_DIS
 */
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000051C )
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_FULLMASK     (u32)0x000334cf
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_10_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_10_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_10_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_9_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_9_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_9_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_8_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_8_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_8_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_7_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_7_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_5_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_5_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_4_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_4_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_AUX_INT_DIS_SERV_REQ_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_REQ_AUX_TRIG
 */
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000520 )
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_FULLMASK     (u32)0x000334cf
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_10_SHIFT   17
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_10_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_10_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_9_SHIFT   16
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_9_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_9_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_8_SHIFT   13
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_8_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_8_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_7_SHIFT   12
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_7_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_7_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_6_SHIFT   10
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_6_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_6_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_5_SHIFT   7
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_5_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_5_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_4_SHIFT   6
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_4_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_4_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_3_SHIFT   3
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_3_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_2_SHIFT   2
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_2_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_1_SHIFT   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_1_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_0_SHIFT   0
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_0_WIDTH   1
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_REQ_AUX_TRIG_SERV_REQ_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MB_FATAL
 */
#define PSMX_GLOBAL_REG_MB_FATAL    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000052C )
#define PSMX_GLOBAL_REG_MB_FATAL_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_MB_FATAL_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MB_FATAL_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MB_FATAL_CPU3_SHIFT   2
#define PSMX_GLOBAL_REG_MB_FATAL_CPU3_WIDTH   1
#define PSMX_GLOBAL_REG_MB_FATAL_CPU3_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_MB_FATAL_CPU3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB_FATAL_CPU2_SHIFT   1
#define PSMX_GLOBAL_REG_MB_FATAL_CPU2_WIDTH   1
#define PSMX_GLOBAL_REG_MB_FATAL_CPU2_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_MB_FATAL_CPU2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB_FATAL_CPU1_SHIFT   0
#define PSMX_GLOBAL_REG_MB_FATAL_CPU1_WIDTH   1
#define PSMX_GLOBAL_REG_MB_FATAL_CPU1_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_MB_FATAL_CPU1_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MB1_FAULT_STATUS
 */
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000530 )
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FULLMASK     (u32)0x00007fff
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU3_SHIFT   14
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU3_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU3_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU2_SHIFT   13
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU2_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU2_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU1_SHIFT   12
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU1_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU1_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_CPU1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FT_STATE_SHIFT   10
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FT_STATE_WIDTH   2
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FT_STATE_MASK    (u32)0x00000c00
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FT_STATE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_WDT_EXP_SHIFT   9
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_WDT_EXP_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_WDT_EXP_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_WDT_EXP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_UNCORR_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_UNCORR_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_UNCORR_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_UNCORR_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_VOTER_ERR_SHIFT   7
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_VOTER_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_VOTER_ERR_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_VOTER_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FTL_MISMATCH_SHIFT   4
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FTL_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FTL_MISMATCH_MASK    (u32)0x00000070
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_FTL_MISMATCH_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_RECOVER_SHIFT   3
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_RECOVER_WIDTH   1
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_RECOVER_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_RECOVER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_MISMATCH_SHIFT   0
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_MISMATCH_MASK    (u32)0x00000007
#define PSMX_GLOBAL_REG_MB1_FAULT_STATUS_LS_MISMATCH_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MB2_FAULT_STATUS
 */
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000534 )
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FULLMASK     (u32)0x00007fff
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU3_SHIFT   14
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU3_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU3_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU2_SHIFT   13
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU2_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU2_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU1_SHIFT   12
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU1_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU1_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_CPU1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FT_STATE_SHIFT   10
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FT_STATE_WIDTH   2
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FT_STATE_MASK    (u32)0x00000c00
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FT_STATE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_WDT_EXP_SHIFT   9
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_WDT_EXP_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_WDT_EXP_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_WDT_EXP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_UNCORR_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_UNCORR_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_UNCORR_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_UNCORR_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_VOTER_ERR_SHIFT   7
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_VOTER_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_VOTER_ERR_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_VOTER_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FTL_MISMATCH_SHIFT   4
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FTL_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FTL_MISMATCH_MASK    (u32)0x00000070
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_FTL_MISMATCH_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_RECOVER_SHIFT   3
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_RECOVER_WIDTH   1
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_RECOVER_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_RECOVER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_MISMATCH_SHIFT   0
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_MISMATCH_MASK    (u32)0x00000007
#define PSMX_GLOBAL_REG_MB2_FAULT_STATUS_LS_MISMATCH_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MB3_FAULT_STATUS
 */
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000538 )
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FULLMASK     (u32)0x00007fff
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU3_SHIFT   14
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU3_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU3_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU2_SHIFT   13
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU2_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU2_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU1_SHIFT   12
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU1_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU1_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_CPU1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FT_STATE_SHIFT   10
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FT_STATE_WIDTH   2
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FT_STATE_MASK    (u32)0x00000c00
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FT_STATE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_WDT_EXP_SHIFT   9
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_WDT_EXP_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_WDT_EXP_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_WDT_EXP_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_UNCORR_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_UNCORR_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_UNCORR_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_UNCORR_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_VOTER_ERR_SHIFT   7
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_VOTER_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_VOTER_ERR_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_VOTER_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FTL_MISMATCH_SHIFT   4
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FTL_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FTL_MISMATCH_MASK    (u32)0x00000070
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_FTL_MISMATCH_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_RECOVER_SHIFT   3
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_RECOVER_WIDTH   1
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_RECOVER_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_RECOVER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_MISMATCH_SHIFT   0
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_MISMATCH_WIDTH   3
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_MISMATCH_MASK    (u32)0x00000007
#define PSMX_GLOBAL_REG_MB3_FAULT_STATUS_LS_MISMATCH_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_DEBUG_CTRL
 */
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000540 )
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_EN_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_EN_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_DEBUG_CTRL_EN_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000600 )
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD7_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD7_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD7_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD6_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD6_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD5_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD5_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD4_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD4_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD3_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD3_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD2_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD2_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD1_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD0_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_STATUS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000604 )
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_DEFVAL   (u32)0xfffffff

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD7_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD7_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD7_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD6_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD6_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD5_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD5_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD4_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD4_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD3_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD3_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD2_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD2_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD1_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD0_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_RSVD0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU3_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU2_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU1_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_MASK_APU0_CORE0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN
 */
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000608 )
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD7_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD7_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD7_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD6_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD6_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD5_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD5_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD4_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD4_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD3_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD3_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD2_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD2_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD1_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD0_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_EN_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS
 */
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000060C )
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD7_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD7_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD7_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD6_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD6_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD5_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD5_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD4_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD4_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD3_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD3_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD2_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD2_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD1_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD0_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_DIS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG
 */
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000610 )
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD7_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD7_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD7_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD6_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD6_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD6_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD5_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD5_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD5_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD4_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD4_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD3_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD3_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD2_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD2_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD1_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD1_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD0_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD0_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP0_IRQ_TRIG_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000614 )
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FULLMASK     (u32)0x7fffffff
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FPD_DBG_SHIFT   30
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FPD_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FPD_DBG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_FPD_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_CORESIGHT_SHIFT   29
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_CORESIGHT_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_CORESIGHT_SHIFT   28
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_CORESIGHT_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_CORESIGHT_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_CORESIGHT_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_CORESIGHT_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_CORESIGHT_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_DSU_CORESIGHT_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_DSU_CORESIGHT_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_DSU_CORESIGHT_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_DSU_CORESIGHT_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_DSU_CORESIGHT_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_DSU_CORESIGHT_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_DSU_CORESIGHT_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_DSU_CORESIGHT_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE3_CORESIGHT_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE3_CORESIGHT_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE2_CORESIGHT_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE2_CORESIGHT_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE1_CORESIGHT_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE1_CORESIGHT_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE0_CORESIGHT_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE0_CORESIGHT_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU3_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE3_CORESIGHT_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE3_CORESIGHT_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE2_CORESIGHT_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE2_CORESIGHT_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE1_CORESIGHT_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE1_CORESIGHT_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE0_CORESIGHT_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE0_CORESIGHT_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU2_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE3_CORESIGHT_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE3_CORESIGHT_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE2_CORESIGHT_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE2_CORESIGHT_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE1_CORESIGHT_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE1_CORESIGHT_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE0_CORESIGHT_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE0_CORESIGHT_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU1_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE3_CORESIGHT_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE3_CORESIGHT_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE2_CORESIGHT_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE2_CORESIGHT_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE1_CORESIGHT_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE1_CORESIGHT_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE0_CORESIGHT_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE0_CORESIGHT_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU0_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CLUSTER_DBG_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CLUSTER_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CLUSTER_DBG_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CLUSTER_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CORE_DBG_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CORE_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CORE_DBG_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_STATUS_APU_CORE_DBG_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000618 )
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FULLMASK     (u32)0x7fffffff
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_DEFVAL   (u32)0x7fffffff

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FPD_DBG_SHIFT   30
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FPD_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FPD_DBG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_FPD_DBG_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_CORESIGHT_SHIFT   29
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_CORESIGHT_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_CORESIGHT_SHIFT   28
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_CORESIGHT_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_CORESIGHT_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_CORESIGHT_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_CORESIGHT_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_CORESIGHT_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_DSU_CORESIGHT_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_DSU_CORESIGHT_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_DSU_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_DSU_CORESIGHT_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_DSU_CORESIGHT_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_DSU_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_DSU_CORESIGHT_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_DSU_CORESIGHT_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_DSU_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_DSU_CORESIGHT_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_DSU_CORESIGHT_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_DSU_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE3_CORESIGHT_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE3_CORESIGHT_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE3_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE2_CORESIGHT_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE2_CORESIGHT_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE2_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE1_CORESIGHT_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE1_CORESIGHT_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE0_CORESIGHT_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE0_CORESIGHT_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU3_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE3_CORESIGHT_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE3_CORESIGHT_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE3_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE2_CORESIGHT_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE2_CORESIGHT_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE2_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE1_CORESIGHT_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE1_CORESIGHT_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE0_CORESIGHT_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE0_CORESIGHT_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU2_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE3_CORESIGHT_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE3_CORESIGHT_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE3_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE2_CORESIGHT_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE2_CORESIGHT_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE2_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE1_CORESIGHT_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE1_CORESIGHT_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE0_CORESIGHT_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE0_CORESIGHT_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU1_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE3_CORESIGHT_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE3_CORESIGHT_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE3_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE2_CORESIGHT_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE2_CORESIGHT_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE2_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE1_CORESIGHT_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE1_CORESIGHT_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE1_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE0_CORESIGHT_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE0_CORESIGHT_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU0_CORE0_CORESIGHT_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_B_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_RPU_A_CORE0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CLUSTER_DBG_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CLUSTER_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CLUSTER_DBG_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CLUSTER_DBG_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CORE_DBG_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CORE_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CORE_DBG_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_MASK_APU_CORE_DBG_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN
 */
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000061C )
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FULLMASK     (u32)0x7fffffff
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FPD_DBG_SHIFT   30
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FPD_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FPD_DBG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_FPD_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_CORESIGHT_SHIFT   29
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_CORESIGHT_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_CORESIGHT_SHIFT   28
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_CORESIGHT_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_CORESIGHT_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_CORESIGHT_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_CORESIGHT_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_CORESIGHT_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_DSU_CORESIGHT_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_DSU_CORESIGHT_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_DSU_CORESIGHT_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_DSU_CORESIGHT_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_DSU_CORESIGHT_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_DSU_CORESIGHT_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_DSU_CORESIGHT_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_DSU_CORESIGHT_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE3_CORESIGHT_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE3_CORESIGHT_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE2_CORESIGHT_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE2_CORESIGHT_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE1_CORESIGHT_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE1_CORESIGHT_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE0_CORESIGHT_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE0_CORESIGHT_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU3_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE3_CORESIGHT_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE3_CORESIGHT_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE2_CORESIGHT_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE2_CORESIGHT_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE1_CORESIGHT_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE1_CORESIGHT_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE0_CORESIGHT_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE0_CORESIGHT_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU2_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE3_CORESIGHT_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE3_CORESIGHT_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE2_CORESIGHT_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE2_CORESIGHT_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE1_CORESIGHT_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE1_CORESIGHT_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE0_CORESIGHT_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE0_CORESIGHT_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU1_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE3_CORESIGHT_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE3_CORESIGHT_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE2_CORESIGHT_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE2_CORESIGHT_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE1_CORESIGHT_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE1_CORESIGHT_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE0_CORESIGHT_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE0_CORESIGHT_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU0_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CLUSTER_DBG_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CLUSTER_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CLUSTER_DBG_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CLUSTER_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CORE_DBG_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CORE_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CORE_DBG_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_EN_APU_CORE_DBG_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS
 */
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000620 )
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FULLMASK     (u32)0x7fffffff
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FPD_DBG_SHIFT   30
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FPD_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FPD_DBG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_FPD_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_CORESIGHT_SHIFT   29
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_CORESIGHT_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_CORESIGHT_SHIFT   28
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_CORESIGHT_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_CORESIGHT_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_CORESIGHT_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_CORESIGHT_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_CORESIGHT_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_DSU_CORESIGHT_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_DSU_CORESIGHT_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_DSU_CORESIGHT_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_DSU_CORESIGHT_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_DSU_CORESIGHT_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_DSU_CORESIGHT_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_DSU_CORESIGHT_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_DSU_CORESIGHT_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE3_CORESIGHT_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE3_CORESIGHT_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE2_CORESIGHT_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE2_CORESIGHT_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE1_CORESIGHT_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE1_CORESIGHT_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE0_CORESIGHT_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE0_CORESIGHT_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU3_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE3_CORESIGHT_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE3_CORESIGHT_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE2_CORESIGHT_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE2_CORESIGHT_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE1_CORESIGHT_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE1_CORESIGHT_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE0_CORESIGHT_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE0_CORESIGHT_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU2_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE3_CORESIGHT_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE3_CORESIGHT_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE2_CORESIGHT_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE2_CORESIGHT_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE1_CORESIGHT_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE1_CORESIGHT_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE0_CORESIGHT_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE0_CORESIGHT_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU1_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE3_CORESIGHT_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE3_CORESIGHT_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE2_CORESIGHT_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE2_CORESIGHT_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE1_CORESIGHT_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE1_CORESIGHT_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE0_CORESIGHT_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE0_CORESIGHT_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU0_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CLUSTER_DBG_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CLUSTER_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CLUSTER_DBG_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CLUSTER_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CORE_DBG_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CORE_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CORE_DBG_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_DIS_APU_CORE_DBG_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG
 */
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000624 )
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FULLMASK     (u32)0x7fffffff
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FPD_DBG_SHIFT   30
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FPD_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FPD_DBG_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_FPD_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_CORESIGHT_SHIFT   29
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_CORESIGHT_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_CORESIGHT_SHIFT   28
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_CORESIGHT_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_CORESIGHT_SHIFT   27
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_CORESIGHT_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_CORESIGHT_SHIFT   26
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_CORESIGHT_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_DSU_CORESIGHT_SHIFT   25
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_DSU_CORESIGHT_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_DSU_CORESIGHT_SHIFT   24
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_DSU_CORESIGHT_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_DSU_CORESIGHT_SHIFT   23
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_DSU_CORESIGHT_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_DSU_CORESIGHT_SHIFT   22
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_DSU_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_DSU_CORESIGHT_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_DSU_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE3_CORESIGHT_SHIFT   21
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE3_CORESIGHT_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE2_CORESIGHT_SHIFT   20
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE2_CORESIGHT_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE1_CORESIGHT_SHIFT   19
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE1_CORESIGHT_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE0_CORESIGHT_SHIFT   18
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE0_CORESIGHT_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU3_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE3_CORESIGHT_SHIFT   17
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE3_CORESIGHT_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE2_CORESIGHT_SHIFT   16
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE2_CORESIGHT_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE1_CORESIGHT_SHIFT   15
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE1_CORESIGHT_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE0_CORESIGHT_SHIFT   14
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE0_CORESIGHT_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU2_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE3_CORESIGHT_SHIFT   13
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE3_CORESIGHT_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE2_CORESIGHT_SHIFT   12
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE2_CORESIGHT_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE1_CORESIGHT_SHIFT   11
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE1_CORESIGHT_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE0_CORESIGHT_SHIFT   10
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE0_CORESIGHT_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU1_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE3_CORESIGHT_SHIFT   9
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE3_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE3_CORESIGHT_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE3_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE2_CORESIGHT_SHIFT   8
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE2_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE2_CORESIGHT_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE2_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE1_CORESIGHT_SHIFT   7
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE1_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE1_CORESIGHT_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE1_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE0_CORESIGHT_SHIFT   6
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE0_CORESIGHT_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE0_CORESIGHT_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU0_CORE0_CORESIGHT_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_SHIFT   3
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_SHIFT   2
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CLUSTER_DBG_SHIFT   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CLUSTER_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CLUSTER_DBG_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CLUSTER_DBG_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CORE_DBG_SHIFT   0
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CORE_DBG_WIDTH   1
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CORE_DBG_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_WAKEUP1_IRQ_TRIG_APU_CORE_DBG_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000700 )
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FPD_SUPPLY_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FPD_SUPPLY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FPD_SUPPLY_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_FPD_SUPPLY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD4_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD4_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD3_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD3_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD2_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD2_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD1_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD1_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD0_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD0_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_STANDBY_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_STANDBY_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU_STANDBY_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU_STANDBY_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_DBG_RST_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_DBG_RST_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_DBG_RST_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_DBG_RST_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_DBG_RST_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_DBG_RST_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_DBG_RST_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_DBG_RST_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_RST_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_RST_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_RST_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_RST_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_B_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_RST_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_RST_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_RST_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_RST_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_RPU_A_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE3_DBG_RST_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE3_DBG_RST_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE2_DBG_RST_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE2_DBG_RST_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE1_DBG_RST_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE1_DBG_RST_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE0_DBG_RST_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE0_DBG_RST_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU3_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE3_DBG_RST_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE3_DBG_RST_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE2_DBG_RST_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE2_DBG_RST_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE1_DBG_RST_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE1_DBG_RST_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE0_DBG_RST_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE0_DBG_RST_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU2_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE3_DBG_RST_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE3_DBG_RST_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE2_DBG_RST_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE2_DBG_RST_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE1_DBG_RST_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE1_DBG_RST_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE0_DBG_RST_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE0_DBG_RST_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU1_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE3_DBG_RST_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE3_DBG_RST_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE2_DBG_RST_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE2_DBG_RST_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE1_DBG_RST_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE1_DBG_RST_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE0_DBG_RST_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE0_DBG_RST_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_STATUS_APU0_CORE0_DBG_RST_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000704 )
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FPD_SUPPLY_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FPD_SUPPLY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FPD_SUPPLY_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_FPD_SUPPLY_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD4_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD4_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD3_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD3_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD2_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD2_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD1_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD1_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD0_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD0_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RSVD0_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_STANDBY_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_STANDBY_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_STANDBY_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU_STANDBY_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU_STANDBY_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU_STANDBY_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_DBG_RST_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_DBG_RST_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_DBG_RST_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_DBG_RST_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_DBG_RST_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_DBG_RST_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_DBG_RST_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_DBG_RST_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_RST_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_RST_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE1_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_RST_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_RST_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_B_CORE0_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_RST_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_RST_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE1_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_RST_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_RST_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_RPU_A_CORE0_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE3_DBG_RST_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE3_DBG_RST_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE3_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE2_DBG_RST_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE2_DBG_RST_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE2_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE1_DBG_RST_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE1_DBG_RST_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE0_DBG_RST_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE0_DBG_RST_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU3_CORE0_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE3_DBG_RST_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE3_DBG_RST_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE3_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE2_DBG_RST_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE2_DBG_RST_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE2_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE1_DBG_RST_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE1_DBG_RST_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE0_DBG_RST_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE0_DBG_RST_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU2_CORE0_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE3_DBG_RST_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE3_DBG_RST_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE3_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE2_DBG_RST_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE2_DBG_RST_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE2_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE1_DBG_RST_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE1_DBG_RST_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE0_DBG_RST_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE0_DBG_RST_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU1_CORE0_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE3_DBG_RST_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE3_DBG_RST_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE3_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE2_DBG_RST_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE2_DBG_RST_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE2_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE1_DBG_RST_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE1_DBG_RST_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE1_DBG_RST_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE0_DBG_RST_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE0_DBG_RST_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_MASK_APU0_CORE0_DBG_RST_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN
 */
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000708 )
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FPD_SUPPLY_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FPD_SUPPLY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FPD_SUPPLY_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_FPD_SUPPLY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD4_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD4_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD3_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD3_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD2_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD2_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD1_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD1_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD0_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD0_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_STANDBY_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_STANDBY_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU_STANDBY_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU_STANDBY_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_DBG_RST_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_DBG_RST_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_DBG_RST_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_DBG_RST_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_DBG_RST_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_DBG_RST_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_DBG_RST_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_DBG_RST_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_RST_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_RST_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_RST_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_RST_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_B_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_RST_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_RST_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_RST_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_RST_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_RPU_A_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE3_DBG_RST_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE3_DBG_RST_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE2_DBG_RST_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE2_DBG_RST_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE1_DBG_RST_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE1_DBG_RST_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE0_DBG_RST_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE0_DBG_RST_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU3_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE3_DBG_RST_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE3_DBG_RST_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE2_DBG_RST_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE2_DBG_RST_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE1_DBG_RST_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE1_DBG_RST_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE0_DBG_RST_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE0_DBG_RST_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU2_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE3_DBG_RST_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE3_DBG_RST_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE2_DBG_RST_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE2_DBG_RST_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE1_DBG_RST_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE1_DBG_RST_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE0_DBG_RST_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE0_DBG_RST_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU1_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE3_DBG_RST_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE3_DBG_RST_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE2_DBG_RST_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE2_DBG_RST_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE1_DBG_RST_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE1_DBG_RST_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE0_DBG_RST_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE0_DBG_RST_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_EN_APU0_CORE0_DBG_RST_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS
 */
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000070C )
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FPD_SUPPLY_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FPD_SUPPLY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FPD_SUPPLY_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_FPD_SUPPLY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD4_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD4_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD3_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD3_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD2_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD2_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD1_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD1_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD0_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD0_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_STANDBY_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_STANDBY_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU_STANDBY_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU_STANDBY_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_DBG_RST_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_DBG_RST_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_DBG_RST_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_DBG_RST_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_DBG_RST_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_DBG_RST_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_DBG_RST_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_DBG_RST_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_RST_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_RST_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_RST_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_RST_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_B_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_RST_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_RST_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_RST_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_RST_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_RPU_A_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE3_DBG_RST_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE3_DBG_RST_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE2_DBG_RST_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE2_DBG_RST_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE1_DBG_RST_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE1_DBG_RST_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE0_DBG_RST_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE0_DBG_RST_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU3_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE3_DBG_RST_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE3_DBG_RST_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE2_DBG_RST_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE2_DBG_RST_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE1_DBG_RST_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE1_DBG_RST_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE0_DBG_RST_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE0_DBG_RST_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU2_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE3_DBG_RST_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE3_DBG_RST_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE2_DBG_RST_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE2_DBG_RST_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE1_DBG_RST_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE1_DBG_RST_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE0_DBG_RST_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE0_DBG_RST_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU1_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE3_DBG_RST_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE3_DBG_RST_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE2_DBG_RST_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE2_DBG_RST_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE1_DBG_RST_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE1_DBG_RST_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE0_DBG_RST_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE0_DBG_RST_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_DIS_APU0_CORE0_DBG_RST_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG
 */
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000710 )
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FPD_SUPPLY_SHIFT   31
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FPD_SUPPLY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FPD_SUPPLY_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_FPD_SUPPLY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD4_SHIFT   30
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD4_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD4_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD3_SHIFT   29
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD3_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD3_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD2_SHIFT   28
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD2_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD2_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD1_SHIFT   27
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD1_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD1_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD0_SHIFT   26
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD0_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD0_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RSVD0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_STANDBY_SHIFT   25
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_STANDBY_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU_STANDBY_SHIFT   24
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU_STANDBY_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU_STANDBY_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU_STANDBY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_DBG_RST_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_DBG_RST_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_DBG_RST_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_DBG_RST_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_DBG_RST_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_DBG_RST_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_DBG_RST_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_DBG_RST_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_RST_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_RST_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_RST_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_RST_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_B_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_RST_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_RST_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE1_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_RST_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_RST_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_RPU_A_CORE0_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE3_DBG_RST_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE3_DBG_RST_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE2_DBG_RST_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE2_DBG_RST_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE1_DBG_RST_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE1_DBG_RST_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE0_DBG_RST_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE0_DBG_RST_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU3_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE3_DBG_RST_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE3_DBG_RST_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE2_DBG_RST_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE2_DBG_RST_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE1_DBG_RST_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE1_DBG_RST_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE0_DBG_RST_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE0_DBG_RST_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU2_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE3_DBG_RST_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE3_DBG_RST_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE2_DBG_RST_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE2_DBG_RST_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE1_DBG_RST_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE1_DBG_RST_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE0_DBG_RST_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE0_DBG_RST_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU1_CORE0_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE3_DBG_RST_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE3_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE3_DBG_RST_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE3_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE2_DBG_RST_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE2_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE2_DBG_RST_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE2_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE1_DBG_RST_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE1_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE1_DBG_RST_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE1_DBG_RST_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE0_DBG_RST_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE0_DBG_RST_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE0_DBG_RST_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL0_IRQ_TRIG_APU0_CORE0_DBG_RST_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000714 )
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_FULLMASK     (u32)0x00ffffff
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE1_PWRDWN_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE1_PWRDWN_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE0_PWRDWN_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE0_PWRDWN_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_B_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE1_PWRDWN_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE1_PWRDWN_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE0_PWRDWN_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE0_PWRDWN_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_RPU_A_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE3_PWRDWN_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE3_PWRDWN_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE2_PWRDWN_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE2_PWRDWN_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE1_PWRDWN_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE1_PWRDWN_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE0_PWRDWN_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE0_PWRDWN_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU3_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE3_PWRDWN_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE3_PWRDWN_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE2_PWRDWN_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE2_PWRDWN_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE1_PWRDWN_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE1_PWRDWN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE0_PWRDWN_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE0_PWRDWN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU2_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE3_PWRDWN_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE3_PWRDWN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE2_PWRDWN_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE2_PWRDWN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE1_PWRDWN_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE1_PWRDWN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE0_PWRDWN_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE0_PWRDWN_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU1_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE3_PWRDWN_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE3_PWRDWN_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE2_PWRDWN_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE2_PWRDWN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE1_PWRDWN_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE1_PWRDWN_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE0_PWRDWN_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE0_PWRDWN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_STATUS_APU0_CORE0_PWRDWN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000718 )
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_FULLMASK     (u32)0x00ffffff
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_DEFVAL   (u32)0xffffff

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE1_PWRDWN_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE1_PWRDWN_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE0_PWRDWN_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE0_PWRDWN_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_B_CORE0_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE1_PWRDWN_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE1_PWRDWN_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE0_PWRDWN_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE0_PWRDWN_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_RPU_A_CORE0_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CLUSTER_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE3_PWRDWN_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE3_PWRDWN_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE3_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE2_PWRDWN_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE2_PWRDWN_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE2_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE1_PWRDWN_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE1_PWRDWN_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE0_PWRDWN_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE0_PWRDWN_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU3_CORE0_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE3_PWRDWN_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE3_PWRDWN_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE3_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE2_PWRDWN_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE2_PWRDWN_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE2_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE1_PWRDWN_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE1_PWRDWN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE0_PWRDWN_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE0_PWRDWN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU2_CORE0_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE3_PWRDWN_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE3_PWRDWN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE3_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE2_PWRDWN_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE2_PWRDWN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE2_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE1_PWRDWN_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE1_PWRDWN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE0_PWRDWN_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE0_PWRDWN_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU1_CORE0_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE3_PWRDWN_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE3_PWRDWN_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE3_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE2_PWRDWN_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE2_PWRDWN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE2_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE1_PWRDWN_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE1_PWRDWN_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE1_PWRDWN_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE0_PWRDWN_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE0_PWRDWN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_MASK_APU0_CORE0_PWRDWN_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN
 */
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000071C )
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_FULLMASK     (u32)0x00ffffff
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE1_PWRDWN_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE1_PWRDWN_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE0_PWRDWN_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE0_PWRDWN_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_B_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE1_PWRDWN_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE1_PWRDWN_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE0_PWRDWN_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE0_PWRDWN_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_RPU_A_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE3_PWRDWN_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE3_PWRDWN_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE2_PWRDWN_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE2_PWRDWN_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE1_PWRDWN_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE1_PWRDWN_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE0_PWRDWN_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE0_PWRDWN_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU3_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE3_PWRDWN_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE3_PWRDWN_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE2_PWRDWN_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE2_PWRDWN_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE1_PWRDWN_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE1_PWRDWN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE0_PWRDWN_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE0_PWRDWN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU2_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE3_PWRDWN_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE3_PWRDWN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE2_PWRDWN_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE2_PWRDWN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE1_PWRDWN_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE1_PWRDWN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE0_PWRDWN_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE0_PWRDWN_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU1_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE3_PWRDWN_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE3_PWRDWN_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE2_PWRDWN_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE2_PWRDWN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE1_PWRDWN_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE1_PWRDWN_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE0_PWRDWN_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE0_PWRDWN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_EN_APU0_CORE0_PWRDWN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS
 */
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000720 )
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_FULLMASK     (u32)0x00ffffff
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE1_PWRDWN_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE1_PWRDWN_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE0_PWRDWN_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE0_PWRDWN_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_B_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE1_PWRDWN_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE1_PWRDWN_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE0_PWRDWN_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE0_PWRDWN_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_RPU_A_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE3_PWRDWN_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE3_PWRDWN_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE2_PWRDWN_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE2_PWRDWN_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE1_PWRDWN_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE1_PWRDWN_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE0_PWRDWN_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE0_PWRDWN_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU3_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE3_PWRDWN_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE3_PWRDWN_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE2_PWRDWN_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE2_PWRDWN_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE1_PWRDWN_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE1_PWRDWN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE0_PWRDWN_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE0_PWRDWN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU2_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE3_PWRDWN_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE3_PWRDWN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE2_PWRDWN_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE2_PWRDWN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE1_PWRDWN_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE1_PWRDWN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE0_PWRDWN_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE0_PWRDWN_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU1_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE3_PWRDWN_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE3_PWRDWN_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE2_PWRDWN_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE2_PWRDWN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE1_PWRDWN_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE1_PWRDWN_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE0_PWRDWN_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE0_PWRDWN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_DIS_APU0_CORE0_PWRDWN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG
 */
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000724 )
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_FULLMASK     (u32)0x00ffffff
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE1_PWRDWN_SHIFT   23
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE1_PWRDWN_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE0_PWRDWN_SHIFT   22
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE0_PWRDWN_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_B_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE1_PWRDWN_SHIFT   21
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE1_PWRDWN_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE0_PWRDWN_SHIFT   20
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE0_PWRDWN_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_RPU_A_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE3_PWRDWN_SHIFT   15
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE3_PWRDWN_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE2_PWRDWN_SHIFT   14
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE2_PWRDWN_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE1_PWRDWN_SHIFT   13
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE1_PWRDWN_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE0_PWRDWN_SHIFT   12
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE0_PWRDWN_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU3_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE3_PWRDWN_SHIFT   11
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE3_PWRDWN_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE2_PWRDWN_SHIFT   10
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE2_PWRDWN_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE1_PWRDWN_SHIFT   9
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE1_PWRDWN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE0_PWRDWN_SHIFT   8
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE0_PWRDWN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU2_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE3_PWRDWN_SHIFT   7
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE3_PWRDWN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE2_PWRDWN_SHIFT   6
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE2_PWRDWN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE1_PWRDWN_SHIFT   5
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE1_PWRDWN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE0_PWRDWN_SHIFT   4
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE0_PWRDWN_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU1_CORE0_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE3_PWRDWN_SHIFT   3
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE3_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE3_PWRDWN_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE3_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE2_PWRDWN_SHIFT   2
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE2_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE2_PWRDWN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE2_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE1_PWRDWN_SHIFT   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE1_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE1_PWRDWN_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE1_PWRDWN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE0_PWRDWN_SHIFT   0
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE0_PWRDWN_WIDTH   1
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE0_PWRDWN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PWR_CTRL1_IRQ_TRIG_APU0_CORE0_PWRDWN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GPI_STATUS
 */
#define PSMX_GLOBAL_REG_GPI_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000728 )
#define PSMX_GLOBAL_REG_GPI_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GPI_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GPI_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_31_SHIFT   31
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_31_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_30_SHIFT   30
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_30_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_29_SHIFT   29
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_29_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_28_SHIFT   28
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_28_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_27_SHIFT   27
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_27_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_26_SHIFT   26
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_26_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_25_SHIFT   25
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_25_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_24_SHIFT   24
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_24_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_23_SHIFT   23
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_23_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_22_SHIFT   22
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_22_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_21_SHIFT   21
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_21_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_20_SHIFT   20
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_20_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_19_SHIFT   19
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_19_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_18_SHIFT   18
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_18_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_17_SHIFT   17
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_17_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_16_SHIFT   16
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_16_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_15_SHIFT   15
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_15_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_14_SHIFT   14
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_14_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_13_SHIFT   13
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_13_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_12_SHIFT   12
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_12_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_11_SHIFT   11
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_11_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_10_SHIFT   10
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_10_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_9_SHIFT   9
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_9_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_8_SHIFT   8
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_8_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_7_SHIFT   7
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_7_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_6_SHIFT   6
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_6_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_5_SHIFT   5
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_5_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_4_SHIFT   4
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_4_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_3_SHIFT   3
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_3_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_2_SHIFT   2
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_2_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_1_SHIFT   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_1_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_0_SHIFT   0
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_0_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GPI_STATUS_GPI_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GPI_MASK
 */
#define PSMX_GLOBAL_REG_GPI_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000072C )
#define PSMX_GLOBAL_REG_GPI_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GPI_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GPI_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_31_SHIFT   31
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_31_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_30_SHIFT   30
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_30_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_29_SHIFT   29
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_29_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_28_SHIFT   28
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_28_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_27_SHIFT   27
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_27_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_26_SHIFT   26
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_26_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_25_SHIFT   25
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_25_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_24_SHIFT   24
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_24_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_23_SHIFT   23
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_23_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_22_SHIFT   22
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_22_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_21_SHIFT   21
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_21_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_20_SHIFT   20
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_20_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_19_SHIFT   19
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_19_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_18_SHIFT   18
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_18_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_17_SHIFT   17
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_17_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_16_SHIFT   16
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_16_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_15_SHIFT   15
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_15_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_14_SHIFT   14
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_14_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_13_SHIFT   13
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_13_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_12_SHIFT   12
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_12_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_11_SHIFT   11
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_11_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_10_SHIFT   10
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_10_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_9_SHIFT   9
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_9_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_8_SHIFT   8
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_8_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_7_SHIFT   7
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_7_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_6_SHIFT   6
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_6_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_5_SHIFT   5
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_5_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_4_SHIFT   4
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_4_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_3_SHIFT   3
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_3_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_2_SHIFT   2
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_2_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_1_SHIFT   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_1_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GPI_MASK_GPI_0_SHIFT   0
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_0_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GPI_MASK_GPI_0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GPI_EN
 */
#define PSMX_GLOBAL_REG_GPI_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000730 )
#define PSMX_GLOBAL_REG_GPI_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GPI_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GPI_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_31_SHIFT   31
#define PSMX_GLOBAL_REG_GPI_EN_GPI_31_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_30_SHIFT   30
#define PSMX_GLOBAL_REG_GPI_EN_GPI_30_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_29_SHIFT   29
#define PSMX_GLOBAL_REG_GPI_EN_GPI_29_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_28_SHIFT   28
#define PSMX_GLOBAL_REG_GPI_EN_GPI_28_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_27_SHIFT   27
#define PSMX_GLOBAL_REG_GPI_EN_GPI_27_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_26_SHIFT   26
#define PSMX_GLOBAL_REG_GPI_EN_GPI_26_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_25_SHIFT   25
#define PSMX_GLOBAL_REG_GPI_EN_GPI_25_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_24_SHIFT   24
#define PSMX_GLOBAL_REG_GPI_EN_GPI_24_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_23_SHIFT   23
#define PSMX_GLOBAL_REG_GPI_EN_GPI_23_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_22_SHIFT   22
#define PSMX_GLOBAL_REG_GPI_EN_GPI_22_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_21_SHIFT   21
#define PSMX_GLOBAL_REG_GPI_EN_GPI_21_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_20_SHIFT   20
#define PSMX_GLOBAL_REG_GPI_EN_GPI_20_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_19_SHIFT   19
#define PSMX_GLOBAL_REG_GPI_EN_GPI_19_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_18_SHIFT   18
#define PSMX_GLOBAL_REG_GPI_EN_GPI_18_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_17_SHIFT   17
#define PSMX_GLOBAL_REG_GPI_EN_GPI_17_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_16_SHIFT   16
#define PSMX_GLOBAL_REG_GPI_EN_GPI_16_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_15_SHIFT   15
#define PSMX_GLOBAL_REG_GPI_EN_GPI_15_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_14_SHIFT   14
#define PSMX_GLOBAL_REG_GPI_EN_GPI_14_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_13_SHIFT   13
#define PSMX_GLOBAL_REG_GPI_EN_GPI_13_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_12_SHIFT   12
#define PSMX_GLOBAL_REG_GPI_EN_GPI_12_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GPI_EN_GPI_12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_11_SHIFT   11
#define PSMX_GLOBAL_REG_GPI_EN_GPI_11_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GPI_EN_GPI_11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_10_SHIFT   10
#define PSMX_GLOBAL_REG_GPI_EN_GPI_10_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GPI_EN_GPI_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_9_SHIFT   9
#define PSMX_GLOBAL_REG_GPI_EN_GPI_9_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GPI_EN_GPI_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_8_SHIFT   8
#define PSMX_GLOBAL_REG_GPI_EN_GPI_8_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GPI_EN_GPI_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_7_SHIFT   7
#define PSMX_GLOBAL_REG_GPI_EN_GPI_7_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GPI_EN_GPI_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_6_SHIFT   6
#define PSMX_GLOBAL_REG_GPI_EN_GPI_6_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GPI_EN_GPI_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_5_SHIFT   5
#define PSMX_GLOBAL_REG_GPI_EN_GPI_5_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GPI_EN_GPI_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_4_SHIFT   4
#define PSMX_GLOBAL_REG_GPI_EN_GPI_4_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GPI_EN_GPI_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_3_SHIFT   3
#define PSMX_GLOBAL_REG_GPI_EN_GPI_3_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GPI_EN_GPI_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_2_SHIFT   2
#define PSMX_GLOBAL_REG_GPI_EN_GPI_2_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GPI_EN_GPI_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_1_SHIFT   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_1_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GPI_EN_GPI_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_EN_GPI_0_SHIFT   0
#define PSMX_GLOBAL_REG_GPI_EN_GPI_0_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_EN_GPI_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GPI_EN_GPI_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GPI_DIS
 */
#define PSMX_GLOBAL_REG_GPI_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000734 )
#define PSMX_GLOBAL_REG_GPI_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GPI_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GPI_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_31_SHIFT   31
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_31_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_30_SHIFT   30
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_30_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_29_SHIFT   29
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_29_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_28_SHIFT   28
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_28_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_27_SHIFT   27
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_27_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_26_SHIFT   26
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_26_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_25_SHIFT   25
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_25_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_24_SHIFT   24
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_24_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_23_SHIFT   23
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_23_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_22_SHIFT   22
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_22_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_21_SHIFT   21
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_21_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_20_SHIFT   20
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_20_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_19_SHIFT   19
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_19_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_18_SHIFT   18
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_18_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_17_SHIFT   17
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_17_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_16_SHIFT   16
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_16_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_15_SHIFT   15
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_15_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_14_SHIFT   14
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_14_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_13_SHIFT   13
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_13_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_12_SHIFT   12
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_12_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_11_SHIFT   11
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_11_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_10_SHIFT   10
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_10_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_9_SHIFT   9
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_9_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_8_SHIFT   8
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_8_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_7_SHIFT   7
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_7_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_6_SHIFT   6
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_6_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_5_SHIFT   5
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_5_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_4_SHIFT   4
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_4_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_3_SHIFT   3
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_3_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_2_SHIFT   2
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_2_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_1_SHIFT   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_1_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_DIS_GPI_0_SHIFT   0
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_0_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GPI_DIS_GPI_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GPI_TRIG
 */
#define PSMX_GLOBAL_REG_GPI_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000738 )
#define PSMX_GLOBAL_REG_GPI_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GPI_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GPI_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_31_SHIFT   31
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_31_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_30_SHIFT   30
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_30_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_29_SHIFT   29
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_29_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_28_SHIFT   28
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_28_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_27_SHIFT   27
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_27_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_26_SHIFT   26
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_26_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_25_SHIFT   25
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_25_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_24_SHIFT   24
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_24_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_23_SHIFT   23
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_23_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_22_SHIFT   22
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_22_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_21_SHIFT   21
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_21_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_20_SHIFT   20
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_20_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_19_SHIFT   19
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_19_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_18_SHIFT   18
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_18_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_17_SHIFT   17
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_17_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_16_SHIFT   16
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_16_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_15_SHIFT   15
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_15_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_14_SHIFT   14
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_14_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_13_SHIFT   13
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_13_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_12_SHIFT   12
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_12_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_11_SHIFT   11
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_11_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_10_SHIFT   10
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_10_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_9_SHIFT   9
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_9_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_8_SHIFT   8
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_8_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_7_SHIFT   7
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_7_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_6_SHIFT   6
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_6_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_5_SHIFT   5
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_5_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_4_SHIFT   4
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_4_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_3_SHIFT   3
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_3_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_2_SHIFT   2
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_2_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_1_SHIFT   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_1_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_0_SHIFT   0
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_0_WIDTH   1
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GPI_TRIG_GPI_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_MB_TEST
 */
#define PSMX_GLOBAL_REG_PSM_MB_TEST    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000800 )
#define PSMX_GLOBAL_REG_PSM_MB_TEST_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_MB_TEST_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_MB_TEST_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_MB_TEST_TMR_DISABLE_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_MB_TEST_TMR_DISABLE_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_MB_TEST_TMR_DISABLE_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_MB_TEST_TMR_DISABLE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SAFETY_GATE
 */
#define PSMX_GLOBAL_REG_SAFETY_GATE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000804 )
#define PSMX_GLOBAL_REG_SAFETY_GATE_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_SAFETY_GATE_FULLRWMASK  (u32)0x00000007
#define PSMX_GLOBAL_REG_SAFETY_GATE_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_SAFETY_GATE_PSM_LOGCLR_ENABLE_SHIFT   2
#define PSMX_GLOBAL_REG_SAFETY_GATE_PSM_LOGCLR_ENABLE_WIDTH   1
#define PSMX_GLOBAL_REG_SAFETY_GATE_PSM_LOGCLR_ENABLE_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_SAFETY_GATE_PSM_LOGCLR_ENABLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_SAFETY_GATE_LBIST_ENABLE_SHIFT   1
#define PSMX_GLOBAL_REG_SAFETY_GATE_LBIST_ENABLE_WIDTH   1
#define PSMX_GLOBAL_REG_SAFETY_GATE_LBIST_ENABLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_SAFETY_GATE_LBIST_ENABLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_SAFETY_GATE_SCAN_ENABLE_SHIFT   0
#define PSMX_GLOBAL_REG_SAFETY_GATE_SCAN_ENABLE_WIDTH   1
#define PSMX_GLOBAL_REG_SAFETY_GATE_SCAN_ENABLE_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_SAFETY_GATE_SCAN_ENABLE_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_DBG_PWR_ACK
 */
#define PSMX_GLOBAL_REG_DBG_PWR_ACK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000808 )
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FULLMASK     (u32)0x03fbffff
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FULLRWMASK  (u32)0x03fbffff
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FPD_SHIFT   25
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FPD_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE1_SHIFT   24
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE1_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE0_SHIFT   23
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE0_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_B_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE1_SHIFT   22
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE1_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE0_SHIFT   21
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE0_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_RPU_A_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_DSU_SHIFT   20
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_DSU_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_DSU_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_DSU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_DSU_SHIFT   19
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_DSU_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_DSU_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_DSU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_DSU_SHIFT   17
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_DSU_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_DSU_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_DSU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_DSU_SHIFT   16
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_DSU_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_DSU_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_DSU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_DBG_PWR_ACK_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER
 */
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000900 )
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CML_SHIFT   24
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CML_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_MEM_CLEAR_TRIGGER_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MEM_CLEAR_DONE
 */
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000904 )
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CML_SHIFT   24
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CML_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_MEM_CLEAR_DONE_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_MEM_CLEAR_PASS
 */
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000908 )
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CML_SHIFT   24
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CML_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_MEM_CLEAR_PASS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER
 */
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000920 )
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CML_SHIFT   24
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CML_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_SCAN_CLEAR_TRIGGER_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SCAN_CLEAR_DONE
 */
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000924 )
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CML_SHIFT   24
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CML_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_SCAN_CLEAR_DONE_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SCAN_CLEAR_PASS
 */
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000928 )
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FULLMASK     (u32)0x0fffffff
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FPD_SHIFT   27
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FPD_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FPD_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_FPD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_PKI_SHIFT   26
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_PKI_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_PKI_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_PKI_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_MMU_SHIFT   25
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_MMU_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_MMU_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_MMU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CML_SHIFT   24
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CML_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CML_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CML_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_1_SHIFT   23
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_1_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_0_SHIFT   22
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_0_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN3_0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN2_SHIFT   21
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN2_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN1_SHIFT   20
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN1_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_CMN1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CLUSTER_SHIFT   19
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CLUSTER_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CLUSTER_SHIFT   18
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CLUSTER_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CLUSTER_SHIFT   17
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CLUSTER_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CLUSTER_SHIFT   16
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CLUSTER_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CLUSTER_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CLUSTER_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE3_SHIFT   15
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE3_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE2_SHIFT   14
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE2_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE1_SHIFT   13
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE1_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE0_SHIFT   12
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE0_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU3_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE3_SHIFT   11
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE3_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE2_SHIFT   10
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE2_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE1_SHIFT   9
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE1_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE0_SHIFT   8
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE0_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU2_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE3_SHIFT   7
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE3_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE2_SHIFT   6
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE2_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE1_SHIFT   5
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE1_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE0_SHIFT   4
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE0_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU1_CORE0_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE3_SHIFT   3
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE3_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE2_SHIFT   2
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE2_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE1_SHIFT   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE1_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE0_SHIFT   0
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE0_WIDTH   1
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_SCAN_CLEAR_PASS_APU0_CORE0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_LPX_DFX_ERR
 */
#define PSMX_GLOBAL_REG_LPX_DFX_ERR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000950 )
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_FULLMASK     (u32)0x0000001f
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_LPX_DFX_ERR_LPX_SHIFT   4
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_LPX_WIDTH   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_LPX_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_LPX_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUB_SHIFT   3
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUB_WIDTH   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUB_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUA_SHIFT   2
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUA_WIDTH   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUA_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_RPUA_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_LPX_DFX_ERR_IOU_SHIFT   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_IOU_WIDTH   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_IOU_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_IOU_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_LPX_DFX_ERR_OCM_SHIFT   0
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_OCM_WIDTH   1
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_OCM_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_LPX_DFX_ERR_OCM_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SAFETY_CHK
 */
#define PSMX_GLOBAL_REG_SAFETY_CHK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000A00 )
#define PSMX_GLOBAL_REG_SAFETY_CHK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_SAFETY_CHK_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_SAFETY_CHK_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_SAFETY_CHK_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_ECO_1
 */
#define PSMX_GLOBAL_REG_ECO_1    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000A04 )
#define PSMX_GLOBAL_REG_ECO_1_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_1_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_1_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_ECO_1_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_ECO_1_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_ECO_1_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_1_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_ECO_2
 */
#define PSMX_GLOBAL_REG_ECO_2    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00000A08 )
#define PSMX_GLOBAL_REG_ECO_2_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_2_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_2_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_ECO_2_VALUE_SHIFT   0
#define PSMX_GLOBAL_REG_ECO_2_VALUE_WIDTH   32
#define PSMX_GLOBAL_REG_ECO_2_VALUE_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_ECO_2_VALUE_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR1_STATUS
 */
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001000 )
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR1_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR2_STATUS
 */
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001004 )
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR2_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR3_STATUS
 */
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001008 )
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR3_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR4_STATUS
 */
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000100C )
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR4_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR1_TRIG
 */
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001020 )
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR1_TRIG_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR2_TRIG
 */
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001024 )
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR2_TRIG_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR3_TRIG
 */
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001028 )
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR3_TRIG_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_ERR4_TRIG
 */
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000102C )
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_ERR4_TRIG_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001040 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR1_EN
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001044 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001048 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR1_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001050 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR2_EN
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001054 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001058 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR2_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001060 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR3_EN
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001064 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001068 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR3_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001070 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR4_EN
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001074 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS
 */
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001078 )
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_CR_ERR4_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001100 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001104 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001108 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR1_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001110 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001114 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001118 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR2_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001120 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001124 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001128 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR3_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001130 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001134 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS
 */
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001138 )
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_NCR_ERR4_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ1_MASK
 */
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001190 )
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ1_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ1_EN
 */
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001194 )
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ1_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ1_DIS
 */
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011A8 )
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ1_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ2_MASK
 */
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011B0 )
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ2_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ2_EN
 */
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011B4 )
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ2_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ2_DIS
 */
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011B8 )
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ2_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ3_MASK
 */
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011C0 )
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ3_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ3_EN
 */
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011C4 )
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ3_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ3_DIS
 */
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000011C8 )
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ3_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ4_MASK
 */
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001200 )
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ4_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ4_EN
 */
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001204 )
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ4_EN_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSM_IRQ4_DIS
 */
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00001208 )
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSM_IRQ4_DIS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP0_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002000 )
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP0_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP0_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002004 )
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP0_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002008 )
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP0_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000200C )
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP0_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002010 )
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP0_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP1_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002014 )
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP1_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP1_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002018 )
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP1_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000201C )
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP1_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002020 )
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP1_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002024 )
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP1_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP2_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002028 )
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP2_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP2_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000202C )
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP2_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002030 )
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP2_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002034 )
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP2_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002038 )
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP2_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP3_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000203C )
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP3_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP3_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002040 )
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP3_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002044 )
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP3_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002048 )
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP3_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000204C )
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP3_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP4_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002050 )
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP4_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP4_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002054 )
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP4_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002058 )
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP4_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000205C )
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP4_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002060 )
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP4_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP5_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002064 )
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP5_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP5_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002068 )
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP5_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000206C )
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP5_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002070 )
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP5_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002074 )
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP5_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP6_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002078 )
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP6_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP6_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000207C )
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP6_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002080 )
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP6_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002084 )
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP6_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002088 )
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP6_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP7_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000208C )
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP7_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP7_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002090 )
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_DEFVAL   (u32)0xffffffff

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC31_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC30_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC29_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC28_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC27_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC26_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC25_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC24_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC23_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC22_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC21_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC20_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC19_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC18_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC17_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC16_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC15_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC14_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC13_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC12_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC11_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC10_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC9_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC8_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP7_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002094 )
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP7_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00002098 )
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP7_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000209C )
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC31_SHIFT   31
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC31_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC31_MASK    (u32)0x80000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC31_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC30_SHIFT   30
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC30_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC30_MASK    (u32)0x40000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC30_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC29_SHIFT   29
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC29_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC29_MASK    (u32)0x20000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC29_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC28_SHIFT   28
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC28_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC28_MASK    (u32)0x10000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC28_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC27_SHIFT   27
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC27_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC27_MASK    (u32)0x08000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC27_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC26_SHIFT   26
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC26_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC26_MASK    (u32)0x04000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC26_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC25_SHIFT   25
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC25_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC25_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC25_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC24_SHIFT   24
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC24_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC24_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC24_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC23_SHIFT   23
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC23_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC23_MASK    (u32)0x00800000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC23_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC22_SHIFT   22
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC22_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC22_MASK    (u32)0x00400000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC22_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC21_SHIFT   21
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC21_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC21_MASK    (u32)0x00200000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC21_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC20_SHIFT   20
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC20_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC20_MASK    (u32)0x00100000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC20_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC19_SHIFT   19
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC19_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC19_MASK    (u32)0x00080000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC19_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC18_SHIFT   18
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC18_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC18_MASK    (u32)0x00040000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC18_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC17_SHIFT   17
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC17_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC17_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC17_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC16_SHIFT   16
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC16_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC16_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC16_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC15_SHIFT   15
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC15_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC15_MASK    (u32)0x00008000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC15_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC14_SHIFT   14
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC14_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC14_MASK    (u32)0x00004000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC14_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC13_SHIFT   13
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC13_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC13_MASK    (u32)0x00002000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC13_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC12_SHIFT   12
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC12_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC12_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC12_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC11_SHIFT   11
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC11_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC11_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC11_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC10_SHIFT   10
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC10_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC10_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC10_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC9_SHIFT   9
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC9_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC9_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC9_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC8_SHIFT   8
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC8_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC8_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC8_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP7_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS
 */
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000020A0 )
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_FULLMASK     (u32)0x000000ff
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_STATUS_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK
 */
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000020A4 )
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_FULLMASK     (u32)0x000000ff
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_DEFVAL   (u32)0xff

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC7_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC6_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC5_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC4_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC3_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC2_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC1_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_MASK_SRC0_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE
 */
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000020A8 )
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_FULLMASK     (u32)0x000000ff
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_ENABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE
 */
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000020AC )
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_FULLMASK     (u32)0x000000ff
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_DISABLE_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER
 */
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000020B0 )
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_FULLMASK     (u32)0x000000ff
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC7_SHIFT   7
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC7_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC7_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC7_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC6_SHIFT   6
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC6_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC6_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC6_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC5_SHIFT   5
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC5_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC5_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC5_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC4_SHIFT   4
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC4_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC4_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC4_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC3_SHIFT   3
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC3_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC3_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC3_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC2_SHIFT   2
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC2_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC2_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC2_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC1_SHIFT   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC1_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC1_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC1_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC0_SHIFT   0
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC0_WIDTH   1
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_GICP_PSM_IRQ_TRIGGER_SRC0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS
 */
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008000 )
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_CORR_ERR_SRC_REGS_0_SHIFT   0
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_CORR_ERR_SRC_REGS_0_WIDTH   1
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_CORR_ERR_SRC_REGS_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_CORR_ERR_SRC_REGS_CORR_ERR_SRC_REGS_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0
 */
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008004 )
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_FULLMASK     (u32)0x0000007f
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_SHIFT   6
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_SHIFT   5
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_SHIFT   4
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_SHIFT   3
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_SHIFT   2
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_SHIFT   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_SHIFT   0
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_CORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS
 */
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008008 )
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_UNCORR_ERR_SRC_REGS_0_SHIFT   0
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_UNCORR_ERR_SRC_REGS_0_WIDTH   1
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_UNCORR_ERR_SRC_REGS_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_UNCORR_ERR_SRC_REGS_UNCORR_ERR_SRC_REGS_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0
 */
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000800C )
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_FULLMASK     (u32)0x0000007f
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_SHIFT   6
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_SHIFT   5
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_PSMX_POWER_MSERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_SHIFT   4
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_MDM_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_SHIFT   3
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_INTLPX_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_SHIFT   2
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_SHIFT   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTPSMX_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_SHIFT   0
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UNCORR_ERR_SRC_REGS_0_INTLPX_INTPSMX_MERB_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS
 */
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008010 )
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_INTR_SRC_REGS_0_SHIFT   0
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_INTR_SRC_REGS_0_WIDTH   1
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_INTR_SRC_REGS_0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_TOP_LVL_INTR_SRC_REGS_INTR_SRC_REGS_0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTR_SRC_REGS_0
 */
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008014 )
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_FULLMASK     (u32)0x0000007f
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_SHIFT   6
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_UBLAZE_INTPSMX_MERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_PSMX_POWER_MSERB_SHIFT   5
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_PSMX_POWER_MSERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_PSMX_POWER_MSERB_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_PSMX_POWER_MSERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_MDM_SERB_SHIFT   4
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_MDM_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_MDM_SERB_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_MDM_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_INTLPX_SERB_SHIFT   3
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_INTLPX_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_INTLPX_SERB_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_INTLPX_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_SHIFT   2
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_GLOBAL_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_APB_SERB_SHIFT   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_APB_SERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_APB_SERB_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTPSMX_APB_SERB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTLPX_INTPSMX_MERB_SHIFT   0
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTLPX_INTPSMX_MERB_WIDTH   1
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTLPX_INTPSMX_MERB_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTR_SRC_REGS_0_INTLPX_INTPSMX_MERB_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SAFETY_CHK_REG
 */
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008018 )
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_FULLMASK     (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_FULLRWMASK  (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_SAFETY_CHK_REG_SHIFT   0
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_SAFETY_CHK_REG_WIDTH   32
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_SAFETY_CHK_REG_MASK    (u32)0xffffffff
#define PSMX_GLOBAL_REG_SAFETY_CHK_REG_SAFETY_CHK_REG_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_SLV_ERR_REG
 */
#define PSMX_GLOBAL_REG_SLV_ERR_REG    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000801C )
#define PSMX_GLOBAL_REG_SLV_ERR_REG_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_SLV_ERR_REG_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_SLV_ERR_REG_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_SLV_ERR_REG_CTRL_SHIFT   0
#define PSMX_GLOBAL_REG_SLV_ERR_REG_CTRL_WIDTH   1
#define PSMX_GLOBAL_REG_SLV_ERR_REG_CTRL_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_SLV_ERR_REG_CTRL_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ
 */
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00008020 )
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_FULLMASK     (u32)0x000001ff
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_FULLRWMASK  (u32)0x000001ff
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BYTE_SHIFT   4
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BYTE_WIDTH   5
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BYTE_MASK    (u32)0x000001f0
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BYTE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BIT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BIT_WIDTH   4
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BIT_MASK    (u32)0x0000000f
#define PSMX_GLOBAL_REG_INTPSMX_CNTL_PARITY_INJ_PARITY_INJ_BIT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009000 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009008 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_ROUTING_BIT0_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_ROUTING_BIT0_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_ROUTING_BIT0_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ROUTE_BIT_CR_ROUTING_BIT0_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000900C )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009010 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009014 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009034 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009054 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_FULLMASK     (u32)0x000003ff
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_FULLRWMASK  (u32)0x000003ff
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_SHIFT   7
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_SHIFT   6
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_SHIFT   5
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_SHIFT   4
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009074 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009078 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x01010003
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000907C )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009080 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_DEFVAL   (u32)0x30001

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009084 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009088 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000908C )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009090 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009094 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009098 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000909C )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090A0 )
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_UBLAZE_INTPSMX_MERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090A4 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090A8 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_DEFVAL   (u32)0x31f0f

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_PARITY_ERR_AW_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090AC )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090B0 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000090B4 )
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_PSMUBLAZE_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009800 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000980C )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009810 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009874 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009878 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000987C )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009880 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009884 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009888 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000988C )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009890 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009894 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x00009898 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000989C )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x000098A0 )
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_PSMX_POWER_MSERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A000 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A00C )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A010 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A014 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A034 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A054 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_FULLMASK     (u32)0x000003ff
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_FULLRWMASK  (u32)0x000003ff
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_SHIFT   7
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_SHIFT   6
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_R_NOT_READY_SHIFT   5
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_R_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_R_NOT_READY_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_R_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_B_NOT_READY_SHIFT   4
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_B_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_B_NOT_READY_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_B_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_W_NOT_READY_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_W_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_W_NOT_READY_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_W_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_TEST_CR_PARITY_ERR_INJ_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A074 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A078 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x01010003
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MDM_AXI_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A07C )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A080 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_DEFVAL   (u32)0x30001

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A084 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A088 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A08C )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A090 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A094 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A098 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A09C )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0A0 )
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_MDM_SERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0A4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ISR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0A8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_DEFVAL   (u32)0x31f0f

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_POISON_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_B_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_PARITY_ERR_AW_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_RD_SLVERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IMR_WR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0AC )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IER_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0B0 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_IDR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000A0B4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_FULLMASK     (u32)0x00031f0f
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_B_SHIFT   12
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_B_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_B_MASK    (u32)0x00001000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_B_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_R_SHIFT   11
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_R_MASK    (u32)0x00000800
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_W_SHIFT   10
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_W_MASK    (u32)0x00000400
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AR_SHIFT   9
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AR_MASK    (u32)0x00000200
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AW_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AW_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AW_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_PARITY_ERR_AW_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MDM_AXI_INTF_ERR_ITR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B000 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B00C )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B010 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B014 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B034 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B054 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_FULLMASK     (u32)0x000001ff
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_FULLRWMASK  (u32)0x000001ff
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_SHIFT   7
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_SHIFT   6
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_SHIFT   5
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_SHIFT   4
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B074 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B078 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x01010003
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_INTLPX_AXI_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B07C )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B080 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_DEFVAL   (u32)0x30001

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B084 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B088 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B08C )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B090 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B094 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B098 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B09C )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0A0 )
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_INTLPX_SERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0A4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ISR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0A8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_DEFVAL   (u32)0x3000f

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_POISON_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_RD_SLVERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IMR_WR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0AC )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IER_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0B0 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_IDR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000B0B4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_INTLPX_AXI_INTF_ERR_ITR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C000 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C00C )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C010 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C014 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C034 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C054 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_FULLRWMASK  (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_INJ_SLVERR_EN_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_INJ_SLVERR_EN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_APB_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_APB_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_APB_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_APB_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_PARITY_INJ_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_PARITY_INJ_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_PARITY_INJ_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_TEST_CR_PARITY_INJ_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C074 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C078 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x01010003
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_GLOBAL_APB_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C07C )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C080 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_DEFVAL   (u32)0x10001

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_TIMEOUT_APB_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C084 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C088 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C08C )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C090 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C094 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C098 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C09C )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0A0 )
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_GLOBAL_APB_SERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0A4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ISR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0A8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_DEFVAL   (u32)0x10101

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_POISON_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_PARITY_ERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IMR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0AC )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IER_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0B0 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_IDR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000C0B4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_GLOBAL_APB_INTF_ERR_ITR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D000 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D00C )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D010 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D014 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D018 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D034 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D038 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D054 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_FULLRWMASK  (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_INJ_SLVERR_EN_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_INJ_SLVERR_EN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_APB_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_APB_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_APB_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_APB_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_PARITY_INJ_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_PARITY_INJ_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_PARITY_INJ_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_TEST_CR_PARITY_INJ_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D058 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_FULLRWMASK  (u32)0x00000007
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_INJ_SLVERR_EN_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_INJ_SLVERR_EN_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_APB_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_APB_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_APB_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_APB_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_PARITY_INJ_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_PARITY_INJ_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_PARITY_INJ_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_TEST_CR_PARITY_INJ_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D074 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D078 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x03030003
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_ERROR_TYPE_SHIFT   25
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_ERROR_TYPE_MASK    (u32)0x02000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_SHIFT   17
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D07C )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D080 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_DEFVAL   (u32)0x10001

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_TIMEOUT_APB_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D084 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D088 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D08C )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_FULLMASK     (u32)0x00010001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_TIMEOUT_APB_SHIFT   16
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_TIMEOUT_APB_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_TIMEOUT_APB_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_TIMEOUT_APB_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D090 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D094 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D098 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D09C )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0A0 )
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTPSMX_APB_SERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0A4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ISR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0A8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_DEFVAL   (u32)0x10101

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_POISON_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_PARITY_ERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IMR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0AC )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IER_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0B0 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_IDR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0B4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_MISCOMPARETRACE_APB_INTF_ERR_ITR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0B8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ISR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0BC )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_DEFVAL   (u32)0x10101

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_POISON_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_PARITY_ERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IMR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0C0 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IER_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0C4 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_IDR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000D0C8 )
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_FULLMASK     (u32)0x00010101
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_POISON_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_POISON_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_POISON_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_POISON_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_PARITY_ERR_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_PARITY_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_PARITY_ERR_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_PARITY_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTPSMX_PSMUBLAZERAM_APB_INTF_ERR_ITR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E000 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_MAIN_CR_TIMEOUT_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E00C )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_FULLMASK     (u32)0x00000003
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_FULLRWMASK  (u32)0x00000003
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_SHIFT   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_POWER_IDLEREQ_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_SOFT_RST_N_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_SOFT_RST_N_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_SOFT_RST_N_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_CR_SOFT_RST_N_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E010 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E014 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_TYP_CR_ERR_CORR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E034 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_FULLRWMASK  (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_EN_CR_INTF_ERR_EN_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E054 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_FULLMASK     (u32)0x000001ff
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_FULLRWMASK  (u32)0x000001ff
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_SHIFT   8
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_MASK    (u32)0x00000100
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_SHIFT   7
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_MASK    (u32)0x00000080
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_RD_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_SHIFT   6
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_MASK    (u32)0x00000040
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_DECERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_SHIFT   5
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_MASK    (u32)0x00000020
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_WR_INJ_SLVERR_EN_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_SHIFT   4
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_MASK    (u32)0x00000010
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_R_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_B_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_W_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AR_NOT_READY_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_TEST_CR_AXI_AW_NOT_READY_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E074 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_FULLMASK     (u32)0x00000007
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_DEFVAL   (u32)0x7

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_SHIFT   2
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_NO_PENDING_TRANS_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLE_SHIFT   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLE_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLE_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ISO_SR_POWER_IDLEACK_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E078 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_FULLMASK     (u32)0x01010003
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_ERROR_TYPE_SHIFT   24
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_ERROR_TYPE_MASK    (u32)0x01000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_INTF_ERR_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_INTF_ERR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_INTF_ERR_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_IF_INTLPX_INTPSMX_AXI_INTF_ERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_SHIFT   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_TYPE_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_TOP_ERR_SRC_SR_ERB_ERROR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E07C )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ISR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E080 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_DEFVAL   (u32)0x30001

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IMR_MISSION_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E084 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IER_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E088 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_IDR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E08C )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_FULLMASK     (u32)0x00030001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_SHIFT   17
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_WR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_SHIFT   16
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_TIMEOUT_AXI_RD_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_MISSION_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_MISSION_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_MISSION_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_ERR_ITR_MISSION_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E090 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ISR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E094 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_DEFVAL   (u32)0x1

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IMR_LATENT_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E098 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IER_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E09C )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_IDR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR
 */
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0A0 )
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_FULLMASK     (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_LATENT_SHIFT   0
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_LATENT_WIDTH   1
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_LATENT_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_INTLPX_INTPSMX_MERB_INTR_ITR_LATENT_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0A4 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ISR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0A8 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_DEFVAL   (u32)0x3000f

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_R_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_POISON_W_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_RD_SLVERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_DECERR_DEFVAL  (u32)0x1

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IMR_WR_SLVERR_DEFVAL  (u32)0x1

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0AC )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IER_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0B0 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_IDR_WR_SLVERR_DEFVAL  (u32)0x0

/**
 * Register: PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR
 */
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR    ( ( PSMX_GLOBAL_REG_BASEADDR ) + (u32)0x0000E0B4 )
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_FULLMASK     (u32)0x0003000f
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_FULLRWMASK  (u32)0x00000000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_DEFVAL   (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_SHIFT   17
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_MASK    (u32)0x00020000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_R_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_SHIFT   16
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_MASK    (u32)0x00010000
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_POISON_W_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_SHIFT   3
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_MASK    (u32)0x00000008
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_SHIFT   2
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_MASK    (u32)0x00000004
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_RD_SLVERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_SHIFT   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_MASK    (u32)0x00000002
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_DECERR_DEFVAL  (u32)0x0

#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_SHIFT   0
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_WIDTH   1
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_MASK    (u32)0x00000001
#define PSMX_GLOBAL_REG_IF_INTLPX_INTPSMX_AXI_INTF_ERR_ITR_WR_SLVERR_DEFVAL  (u32)0x0


#ifdef __cplusplus
}
#endif

#endif /* XPSMFW_PSMX_GLOBAL_H_ */
